<?php

/**
 * This is the model class for table "gportal_apalines_file".
 *
 * The followings are the available columns in table 'gportal_apalines_file':
 * @property integer $id
 * @property string $href
 * @property integer $size
 * @property integer $width
 * @property integer $height
 * @property string $format
 * @property integer $parent_id
 */
class ApalinesFile extends CActiveRecord
{
    const THUMBNAIL = 0;
    const ORIGINAL  = 1;
    const HIRES     = 2;
	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return ApalinesFile the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'gportal_apalines_file';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('parent_id', 'required'),
			array('size, width, height, parent_id', 'numerical', 'integerOnly'=>true),
			array('href', 'length' , 'max'=>255),
            array('format', 'length' , 'max'=>5),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, href, size, width, height, format, parent_id', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'href' => 'Href',
			'size' => 'Size',
			'width' => 'Width',
			'height' => 'Height',
			'format' => 'Format',
			'parent_id' => 'Parent',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('href',$this->href,true);
		$criteria->compare('size',$this->size);
		$criteria->compare('width',$this->width);
		$criteria->compare('height',$this->height);
		$criteria->compare('format',$this->format);
		$criteria->compare('parent_id',$this->parent_id);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
    /**
     * Function removes multimedia files before record is deleted
     */
    public function beforeDelete() {

        $channel = Channel::$instance;
        $file    = $this->href;

        if (isset($channel)) {
            if (file_exists($channel->makeImagesPath($file))) {
                unlink($channel->makeImagesPath($file));
                Yii::log("{$file} done  delete image", 'info');
            } elseif(file_exists($channel->makeVideosPath($file))) {
                Yii::log("{$file} done  delete video", 'info');
                unlink($channel->makeVideosPath($file));
            }
        }
        return parent::beforeDelete();
    }
}
