<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Class for loading content from File System Content Renderer
 *
 * @property string $folder
 */
class FileSystemContentSource extends ContentSource
{
    /**
     * @var string folder which contained File System Content Renderer content
     */
    public $contentFolder;


    /**
     * Return array of attributes which will be requested from API
     *
     * @return array array of attributes
     */
    public function getRequestedAttributes()
    {
        if ($this->usePersonalisation) {
            return $this->personalisationFields;
        } else {
            return array();
        }
    }

    /**
     * Component initial function
     *
     * @return void
     */
    public function init()
    {

    }

    /**
     * Get content absolute path.
     *
     * @param string $path content path
     *
     * @return bool|\Content|mixed local file location
     */
    public function getContent($path)
    {
        $contentFile = $this->contentFile($path);
        if ($this->fileAccessible($contentFile, $this->contentFolder)) {
            $content = new Content($this, $path);
            return $content;
        }

        return false;
    }


    /**
     * Get the content object by its content ID
     *
     * @param integer  Content ID without type, e.g.: 123
     * @param string   $path content path
     */
    public function getContentById($contentId, $path = null)
    {
        throw new Exception("Not implemented");
    }

    public function getStartPage($folderPath, $locale)
    {
        return false;
    }
    /**
     * Get content file location
     *
     * @param string $path content path
     *
     * @return string
     */
    public function contentFile($path)
    {
        return $this->contentFolder . $path;
    }
}
