<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Controller which recieve all requests to Content Repository
 *
 * All request for pages which should be rendered(html, php) handled by actionDynamic()
 * All other requests, for static content (img, doc, pdf, etc.) handled by actionStatic()
 * Css and js files also handled by actionStatic() if they are not wrapped
 * in Yii::app()->clientScript->registerCssFile() or Yii::app()->clientScript->registerScriptFile().
 * If they are wrapped then they will compressed and stored in /assets folder.
 *
 * To handle request to specific node from content repository add these lines into routers.php
 * 'rules' => array(
'<path:nodename\/.+\.(php|html)>' => 'contentRepository/dynamic',
'<path:nodename\/.+>' => 'contentRepository/static',
),
 * nodename - name of node(for exaple - gportal)
 *
 */
class RendererController extends BaseController
{
    /**
     * Default action
     *
     * @return void
     */
    function actionIndex()
    {
        $this->redirect(Yii::app()->createUrl(Yii::app()->getModule('user')->homePageUrl));
    }

    /**
     * Action which render all static content: js, css, doc, pdf files etc.
     *
     * @throws CHttpException
     * @throws Exception
     *
     * @return void
     */
    function actionStatic()
    {
        if (Yii::app()->request->getParam('path',false)) {
            $path = '/' . Yii::app()->request->getParam('path','');
            $content = $this->_getContentSource()->getContent($path);

            if ($content) {
                if ($content->hasAccess(Yii::app()->user)) {
                    if ($content->isPartial()){
                        $realPath = $content->getFile();
                        $attributes = $content->getAttributes();
                        return $this->_partialLoad($realPath, $attributes['mimetype']);
                    }

                    $filePath = $content->getFile();
                    $fileContent = file_get_contents($filePath);

                    HeadersHelper::sendHttpheaders(HeadersHelper::buildStaticFileHeaders($filePath));
                    HeadersHelper::sendCacheHttpHeaders($this->_getContentSource()->getBrowserCacheTimeByFileName($filePath));

                    if ($this->_getContentSource()->useDynamicFileCaching) {
                        $pathinfo = pathinfo($path,PATHINFO_EXTENSION);
                        $extension = "";

                        if (isset($pathinfo)) {
                            $extension = $pathinfo;
                        }

                        if (in_array($extension,$this->_getContentSource()->dynamicFileCachingExtensions)) {
                            $cacheTimestamp = Yii::app()->cache->get(md5($path."_filecache"));
                            $cacheDeleteTimestamp = Yii::app()->cache->get(md5($path."_deletefilecache"));
                            if ($cacheTimestamp !== false) {
                                $cacheDeleteTimestamp = Yii::app()->cache->set(md5($path."_deletefilecache"),time(),$this->_getContentSource()->dynamicFileCachingDeleteDuration);
                            }

                            if (isset($pathinfo)) {
                                $extension = $pathinfo;
                            }

                            if (in_array($extension, $this->_getContentSource()->dynamicFileCachingExtensions)) {
                                $cacheTimestamp = Yii::app()->cache->get(md5($path . "_filecache"));
                                $cacheDeleteTimestamp = Yii::app()->cache->get(md5($path . "_deletefilecache"));
                                if ($cacheTimestamp !== false) {
                                    $cacheDeleteTimestamp = Yii::app()->cache->set(md5($path . "_deletefilecache"), time(), $this->_getContentSource()->dynamicFileCachingDeleteDuration);
                                }

                                if ($cacheDeleteTimestamp == false) {
                                    unlink($this->_getContentSource()->getContentSource()->contentFile($path));
                                }

                                Yii::app()->cache->set(md5($path . "_filecache"), time(), $this->_getContentSource()->dynamicFileCachingNoCacheDuration);
                            }
                        }

                        echo $fileContent;
                        die;
                    }
                } else {
                    $this->_noAccess($content);
                }
            } else {
                if (YII_DEBUG) {
                    throw new Exception('Path not exists: ' . $this->_getContentSource()->contentSource->contentFile($path));
                } else {
                    throw new CHttpException(404);
                }
            }
        }
    }

    /**
     * returns content using partial content headers (206)
     * original source: https://gist.github.com/codler/3906826
     */
    private function _partialLoad($file, $mimetype){
        $fp = @fopen($file, 'rb');
        $size   = filesize($file); // File size
        $length = $size;           // Content length
        $start  = 0;               // Start byte
        $end    = $size - 1;       // End byte
        header("Content-type: $mimetype");
        header("Accept-Ranges: 0-$length");
        if (isset($_SERVER['HTTP_RANGE'])) {
            $c_start = $start;
            $c_end   = $end;
            list(, $range) = explode('=', $_SERVER['HTTP_RANGE'], 2);
            if (strpos($range, ',') !== false) {
                header('HTTP/1.1 416 Requested Range Not Satisfiable');
                header("Content-Range: bytes $start-$end/$size");
                exit;
            }
            if ($range == '-') {
                $c_start = $size - substr($range, 1);
            }else{
                $range  = explode('-', $range);
                $c_start = $range[0];
                $c_end   = (isset($range[1]) && is_numeric($range[1])) ? $range[1] : $size;
            }
            $c_end = ($c_end > $end) ? $end : $c_end;
            if ($c_start > $c_end || $c_start > $size - 1 || $c_end >= $size) {
                header('HTTP/1.1 416 Requested Range Not Satisfiable');
                header("Content-Range: bytes $start-$end/$size");
                exit;
            }
            $start  = $c_start;
            $end    = $c_end;
            $length = $end - $start + 1;
            fseek($fp, $start);
            header('HTTP/1.1 206 Partial Content');
        }
        header("Content-Range: bytes $start-$end/$size");
        header("Content-Length: ".$length);
        $buffer = 1024 * 8;
        while(!feof($fp) && ($p = ftell($fp)) <= $end) {
            if ($p + $buffer > $end) {
                $buffer = $end - $p + 1;
            }
            set_time_limit(0);
            echo fread($fp, $buffer);
            flush();
        }
        fclose($fp);
        exit();
    }

    /**
     * Action which render all dynamic content: php, html files
     *
     * @throws CHttpException
     * @throws Exception
     *
     * @return void
     */
    function actionDynamic()
    {
        if (Yii::app()->request->getParam('path',false)) {
            $path = '/' . Yii::app()->request->getParam('path','');

            $content = $this->_getContentSource()->getContent($path);
            if ($content) {
                if ($content->hasAccess(Yii::app()->user)) {
                     //if pdf = true and pdf creation by direct url is allowed, call createPdf
                     if (Yii::app()->request->getParam('pdf',false) && Yii::app()->getModule('pdf')->createByParam){
                          Yii::app()->getModule('pdf')->createPdf();
                     } else {
                          $content = $this->renderFile($content->getFile(), null, true);
                          echo $this->processOutput($content);
                     }
                } else {
                    $this->_noAccess($content);
                }
            } else {
                if (YII_DEBUG) {
                    throw new Exception('Path not exists: ' . $this->_getContentSource()->contentSource->contentFile($path));
                } else {
                    throw new CHttpException(404);
                }
            }
        }
    }

    /**
     * Handle folder start page. In case user make request to folder like /Content.Node/service/
     *
     * @return void
     * @author Me
     **/
    public function actionStartPage()
    {
        if($this->module->getContentSource() instanceof FileSystemContentSource) {
            throw new CHttpException(404);
        }

        $folder = Yii::app()->request->getParam('folder', '').'/';
        $startPage = $this->module->getContentSource()->getStartPage($folder, substr(Yii::app()->locale->id, 0, 2));

        if($startPage === false) {
            $startPage = '/' . Yii::app()->request->getParam('path','') . $folder . $this->module->startPageFallback;
        }

        if($this->module->startPageDynamic) {
            $_GET['path'] = substr($startPage,1);
            $this->actionDynamic();
        } else {
            $this->redirect($startPage);
        }
    }
    /**
     * Sets corresponding messages and performs redirect if user don't have access to requested content
     *
     * @param string $path content path
     *
     * @return void
     */
    private function _noAccess($content)
    {
        if (Yii::app()->user->isGuest) {
            $message = UserModule::t('Please login to access required content.');
        } else {
            $message = UserModule::t("You don't have enough rights to access this content. You should have such permissions: {permissions}", array('{permissions}'=> implode(', ', $content->getPersonalisationAttributes())));
        }

        Yii::app()->user->setFlash(
            Yii::app()->getModule('user')->userLoginWidgetFlashError,
            $message
        );
        Yii::app()->user->setReturnUrl($content->getPath());
        $this->redirect(Yii::app()->getModule('user')->loginUrl);
    }

    public function processOutput($output){
        $processed = parent::processOutput($output);
        $headPosition = strpos($processed, '<head>');
        $is_head = false;
        preg_match("/<head>(?P<head_content>.*)<\/head>/s", $processed, $is_head);
        if(!empty($is_head)){
            $is_head = trim($is_head['head_content']);
            $is_head = strlen($is_head) ? true: false;
        }else{
            $is_head = false;
        }
        if($headPosition && $is_head){
            $processed = substr_replace($processed, AppHelper::footprint('<!-- generated by Gentics Portal.Node -->'), $headPosition, 0);
            if(isset($_GET['error'])){
                $error_code = $_GET['error'];
                $error_msg = self::getStatusText($error_code);
                header("HTTP/1.1 $error_code $error_msg");
            }
        }
        return $processed;
    }

    static function getStatusText($code){

        $status_codes = array (

            100 => 'Continue',
            101 => 'Switching Protocols',
            102 => 'Processing',
            200 => 'OK',
            201 => 'Created',
            202 => 'Accepted',
            203 => 'Non-Authoritative Information',
            204 => 'No Content',
            205 => 'Reset Content',
            206 => 'Partial Content',
            207 => 'Multi-Status',
            300 => 'Multiple Choices',
            301 => 'Moved Permanently',
            302 => 'Found',
            303 => 'See Other',
            304 => 'Not Modified',
            305 => 'Use Proxy',
            307 => 'Temporary Redirect',
            400 => 'Bad Request',
            401 => 'Unauthorized',
            402 => 'Payment Required',
            403 => 'Forbidden',
            404 => 'Not Found',
            405 => 'Method Not Allowed',
            406 => 'Not Acceptable',
            407 => 'Proxy Authentication Required',
            408 => 'Request Timeout',
            409 => 'Conflict',
            410 => 'Gone',
            411 => 'Length Required',
            412 => 'Precondition Failed',
            413 => 'Request Entity Too Large',
            414 => 'Request-URI Too Long',
            415 => 'Unsupported Media Type',
            416 => 'Requested Range Not Satisfiable',
            417 => 'Expectation Failed',
            422 => 'Unprocessable Entity',
            423 => 'Locked',
            424 => 'Failed Dependency',
            426 => 'Upgrade Required',
            500 => 'Internal Server Error',
            501 => 'Not Implemented',
            502 => 'Bad Gateway',
            503 => 'Service Unavailable',
            504 => 'Gateway Timeout',
            505 => 'HTTP Version Not Supported',
            506 => 'Variant Also Negotiates',
            507 => 'Insufficient Storage',
            509 => 'Bandwidth Limit Exceeded',
            510 => 'Not Extended'
        );

        $msg = isset($status_codes[$code])?$status_codes[$code]:'An unknown error';

        return $msg;
    }

    /**
     * Register css file in page
     *
     * @param string $file      filename
     * @param null   $timestamp last update timestamp of this file
     * @param bool   $combine   if combine and compress this file
     * @param string $media     media that the CSS code should be applied to. If empty, it means all media types.
     *
     * @return void
     */
    public function getCss($file, $timestamp = null, $combine = true, $media = '')
    {
        Yii::app()->clientScript->registerCssFile($file, $media, $timestamp, $combine, true, true);
    }

    /**
     * Register js file in page
     *
     * @param string $file      filename
     * @param null   $timestamp last update timestamp of this file
     * @param bool   $combine   if combine and compress this file
     *
     * @return void
     */
    public function getJs($file, $timestamp = null, $combine = true)
    {
        Yii::app()->clientScript->registerScriptFile($file, CClientScript::POS_HEAD,
                array(), $timestamp, $combine, true, true);
    }

    /**
     * Register general javascript library which is used by page and widgets and which should be of the same version in many palces.
     * For example jQuery library
     *
     * @param string $file filename
     *
     * @return void
     */
    public function getCoreJs($file)
    {
        Yii::app()->clientScript->registerCoreScript($file, true);
    }

    /**
     * Get current content source for Static or Dynamic content renderer
     *
     * @return ContentSourceModule
     */
    private function _getContentSource()
    {
        return Yii::app()->getModule('contentSource');
    }
}
