(function($){
	
	/**
	 * Collection of elements helpers
	 * @namespace ElementsHelper
	 */
	$.fg.ElementsHelper = {};
	
	/**
	 * Used to build GUI in {@link $.fg.ElementEditor} area to construct label => value elements properties
	 * @class listBuilder
	 */
	$.fg.ElementsHelper.listBuilder = $.Class.extend(
	/** @lends $.fg.ElementsHelper.listBuilder */
	{
			/**
			 * listBuilder area DOM node
			 * @type {object}
			 */
			object: null,
			/**
			 * Reference to element property
			 * @type {object}
			 */
			propertyRecipient: null,
		
		/**
		 * Initialize listBuilder
		 * @param {object} propertyRecipient reference to element property
		 * @return {void}
		 */
		init: function(propertyRecipient)
		{
			this.propertyRecipient = propertyRecipient;
		},
		
		/**
		 * Render listBuilder area
		 * @return {object} DOM node
		 */
		render: function()
		{
			var property = this.propertyRecipient;
			var builder = $( "<div>" ).addClass( "listbuilder" );
			this.object = builder;
			var title = $( "<h4>" );
			var buttonAdd = $( "<a>" ).addClass( "button-add" ).attr( "href", "#" ).click( $.proxy( this.addHandler, this ) );
			var ul = $( "<ul>" );
			title.append(
				$( "<div>" ).addClass( "title values" ).text( "Values" ),
				$( "<div>" ).addClass( "title labels" ).text( "Labels" ),
				buttonAdd
			);
			builder.append( title );
			builder.append( ul );
			//$.fg.ElementEditor.object.find( ".content .builder" ).append( builder );
			
			if(property && typeof property === "object" && property.length) {
				$(property).each($.proxy(function(k, v) {
					this.addRow( v.value, v.label );
				}, this));
			} else {
				this.addRow();
			}
			
			ul.sortable({
				handler: "a.button-move",
				axis: "y",
				update: $.proxy( this.onSortUpdate, this )
			});
			
			return builder;
		},
		
		/**
		 * Add row to the listBuilder to add and edit new array item
		 * @param {string} value default value for value input
		 * @param {string} label default value for label input
		 * @return {void}
		 */
		addRow: function(value, label)
		{
			var ul = this.object.find( "ul" );
			var li = $( "<li>" );
			var inputValue = $( "<input>" )
				.attr( "type", "text" )
				.val( value )
				.change( $.proxy( this.onInputChange, this ) );
			var inputLabel = $( "<input>" )
				.attr( "type", "text" )
				.val( label )
				.change( $.proxy( this.onInputChange, this ) );
			var buttonMove = $( "<a>" ).addClass( "button-move" ).attr( "href", "#" ).click( $.proxy( this.sortHandler, this ) );
			var buttonDelete = $( "<a>" ).addClass( "button-delete" ).attr( "href", "#" ).click( $.proxy( this.deleteHandler, this ) );
			li.append( $( "<div>" ).addClass( "value" ).append( inputValue ) );
			li.append( $( "<div>" ).addClass( "label" ).append( inputLabel ) );
			li.append( buttonMove );
			li.append( buttonDelete );
			ul.append( li );
		},
		
		/**
		 * Remove row from listBuilder
		 * @return {void}
		 */
		deleteRow: function(index)
		{
			var ul = this.object.find( "ul" );
			ul.find( "li" ).eq(index).remove();
			this.updateArray();
			if(ul.find( "li" ).length == 0) {
				this.addRow();
			}
		},
		
		/**
		 * Update array depending on existing listBuilder rows and input values<br />
		 * Also rerender element in the design area by calling {@link $.fg.ElementEditor.updateElement}
		 * @return {void}
		 */
		updateArray: function()
		{
			var rows = $.fg.ElementEditor.object.find( "ul li" );
			// clear property items and not lose the reference to this property
			while(this.propertyRecipient.length) {
				this.propertyRecipient.pop();
			}
			$.each( rows, $.proxy(function(k, v) {
				var inputs = $(v).find( "input" );
				var value = $(inputs[0]).val();
				var label = $(inputs[1]).val();
				if(value === '' && label === '') {
					return;
				}
				this.propertyRecipient.push( { value: value, label: label } );
			}, this) );
			$.fg.ElementEditor.updateElement();
		},
		
		/**
		 * Handler of sort update of listBuilder rows
		 * @param {event} event
		 * @param {object} ui
		 * @return {void}
		 */
		onSortUpdate: function(event, ui)
		{
			var propertyName = ui.item.closest( ".box" ).attr( "property-name" );
			this.updateArray( propertyName );
		},
		
		/**
		 * Handler of row input change
		 * @param {event} event
		 * @return {void}
		 */
		onInputChange: function(event)
		{
			this.updateArray();
		},
		
		/**
		 * Handler of click on sort button
		 * @param {event} event
		 * @return {void}
		 */
		sortHandler: function(event)
		{
			event.preventDefault();
		},
		
		/**
		 * Handler of click on delete button
		 * @param {event} event
		 * @return {void}
		 */
		deleteHandler: function(event)
		{
			event.preventDefault();
			var t = $(event.target);
			this.deleteRow( t.closest( "li" ).index() );
		},
		
		/**
		 * Handler of click on add button
		 * @param {event} event
		 * @return {void}
		 */
		addHandler: function(event)
		{
			event.preventDefault();
			this.addRow();
		}
		
	});
	
})(jQuery);