<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Like module class file.
 */
class LikeModule extends CWebModule
{

    /**
     * @var string
     */
    public $defaultController = 'like';

    /**
     * Action for liking/unliking content
     * @var likeAction
     */
    public $likeAction;

    /**
     * ID of the content that has to be liked/unliked
     * @var contentId
     */
    public $contentId;

    /**
     * Defines how many likers should be shown in div on mouseover. Default is 10
     * @var lastLikesQty
     */
    public $lastLikesQty = 10;

    /**
     * Defines how many likers should be shown in div on mouseover. Default is 3
     * @var topLikesQty
     */
    public $topLikesQty = 3;

    /**
     * @var int the number of seconds in which the cached value will expire. 0 means never expire.
     */
    public $cacheTime = 60;

    private $_assetsUrl;
    /**
     * page contained UserLoginWidget
     *
     * @return mixed
     */
    public function getLoginUrl()
    {
        return is_array(Yii::app()->getModule('user')->loginUrl) ? Yii::app()->getModule('user')->loginUrl[0] : Yii::app()->getModule('user')->loginUrl;
    }

    /**
     * Initialize widget. Import like models.
     *
     * @return void
     */
    public function init()
    {
        // import the module-level models and components
        $this->setImport(
            array(
                'like.models.*',
            )
        );
    }

    /**
     * Translates a message to the specified language.
     *
     * @param string $str    message
     * @param array  $params params
     * @param string $dic    dictionary
     *
     * @return string
     */
    public static function t($str = '', $params = array(), $dic = 'like')
    {
        return Yii::t("LikeModule." . $dic, $str, $params);
    }


    /**
     * Returns path to assets for current module
     *
     * @return string
     */
    public function getAssetsUrl()
    {
        if (!$this->_assetsUrl) {
            $this->_assetsUrl = Yii::app()->assetManager->publish(Yii::getPathOfAlias('like.views.asset'), true);
        }
        return $this->_assetsUrl;
    }
}
