<?php

/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * VoteModule
 * This module represent voting system which manage by questions and answers and return statistics of answers 
 *
 *
**/
class VoteModule extends CWebModule
{
    public $debug = false;
    public $_assetsUrl;

    public function getLoginUrl()
    {
        return Yii::app()->getModule('user')->loginUrl;
    }
    
    public function init()
    {
        $this->setImport(array(
                    'vote.models.*'
                    )); 
    }

    /**
     * Translates a message to the specified language.
     *
     * @param string $str    message
     * @param array  $params params
     * @param string $dic    dictionary
     *
     * @return string
     */
    public static function t($str = '', $params = array(), $dic = 'core')
    {
        return Yii::t("VoteModule." . $dic, $str, $params);
    }

    /**
     * Get current user id
     *
     * @return string
     **/
    public function getUserId()
    {
        return Yii::app()->user->isGuest ? Yii::app()->session->getSessionId() : Yii::app()->user->id;
    }

    /**
     * Checks if user already answered for question
     *
     * @return boolean
     **/
    public function isUserVoted($questionId, $userId = null) 
    {
        if($userId == null) {
            $userId = $this->getUserId();
        }
        return Yii::app()->user->getState('vote-voted-'.$questionId) === true;
    }

    /**
     * Set status of question as answered
     *
     * @return void
     **/
    public function setUserVoted($questionId, $userId = null) 
    {
        if($userId == null) {
            $userId = $this->getUserId();
        }
        Yii::app()->user->setState('vote-voted-'.$questionId, true);
    }

    /**
     * Unset status of question as answered
     *
     * @return void
     **/
    public function setUserUnvoted($questionId, $userId = null) 
    {
        if($userId == null) {
            $userId = $this->getUserId();
        }
        Yii::app()->user->setState('vote-voted-'.$questionId, false);
    }

    public function isAdmin()
    {
        return true;
    }

    /**
     * Publish module assets and return url
     *
     * @return string
     */
    public function getAssetsUrl()
    {
        if (!$this->_assetsUrl) {
            $this->_assetsUrl = Yii::app()->assetManager->publish(Yii::getPathOfAlias('vote.assets'), true, -1, $this->debug);
        }
        return $this->_assetsUrl;
    }
}
