<?php
/**
 * PortalLoginPlugin for phplist
 *
 * The plugin provides admin authentication to phplist using portal user personalisation attributes.
 * When portal user is logged in and in the session variable a certain permission is set, login as well to phplist.
 * 
 * This file should be located in lists/admin/plugins folder. 
 * The portal has to write the yii prefix into the session, so that personalisation attributes 
 * in the session can be accessed here.
 * 
 * @category    phplist
 * @package     PortalLoginPlugin
 */

/**
 * Registers the plugin with phplist
 */
class PortalLoginPlugin extends phplistPlugin
{
    /*
     * Inherited Variables
     */
    public $name = 'Portal Login Plugin';
    public $description = 'Provides authentication to phplist using portal user personalisation attributes.';
    public $version = 1.0;
    
    /**
     * portal permissions of phplist admins
     * @var array
     */
    private $adminPermissions = array('phplistadmin');
    
    public function __construct()
    {
        $this->coderoot = dirname(__FILE__) . '/' . __CLASS__ . '/';
        parent::__construct();
    }
    
    public function activate()
    {
        if (!isset($_SESSION['adminloggedin']))
            self::login();
    }
    
    /**
     * logs in a user if portal permission for phplist is set
     * @return [type] [description]
     */
    public function login()
    {
        global $tables;
        
        // yii prefix session variable set during successful login
        if (isset($_SESSION['yiiPrefix'])) {
            $yiiPrefix = $_SESSION['yiiPrefix'];
            if (isset($_SESSION[$yiiPrefix . 'personalisation']) && !empty(array_intersect($this->adminPermissions, $_SESSION[$yiiPrefix . 'personalisation']))) {
                // user is logged in and has permission                 
                $login     = $_SESSION[$yiiPrefix . '__name'];
                $mail      = $_SESSION[$yiiPrefix . 'email'];
                // all users with right permission are super admins by default
                $superuser = 1;
                
                $row = Sql_Fetch_Row_Query(sprintf("SELECT id, privileges
                    FROM {$tables['admin']}
                    WHERE loginname = '%s'", sql_escape($login)));
                
                if ($row) {
                    list($id, $privileges) = $row;
                    
                    $update = Sql_Query(sprintf("UPDATE {$tables['admin']} SET
                        email = '%s',
                        superuser = %s,
                        disabled = 0
                        WHERE id=%s", sql_escape($mail), $superuser, $id));
                    
                    if (!$update) {
                        die(Fatal_Error(s("Fail to update user informations in database : %s", Sql_Error())));
                    }
                } else {
                    $insert = Sql_Query(sprintf("INSERT INTO {$tables['admin']}
                        (loginname,email,superuser,disabled)
                        VALUES
                        ('%s','%s',%s,0)", sql_escape($login), sql_escape($mail), $superuser));
                    if (!$insert) {
                        die(Fatal_Error(s("Fail to create user in database : %s", Sql_Error())));
                    }
                    
                    $id = Sql_Insert_Id();
                }
                
                $_SESSION['adminloggedin'] = $_SERVER["REMOTE_ADDR"];
                $_SESSION['logindetails']  = array(
                    'adminname' => $login,
                    'id' => $id,
                    'superuser' => $superuser
                );
                
                if ($privileges) {
                    $_SESSION['privileges'] = unserialize($privileges);
                }
                
                return true;
                
            } else {
                // user is logged in but has no permission.
            }
            
        } else {
            // user is guest or has no permission.
        }
        
        return true;
    }
    
    //When user logs out redirect them to the CAS logout page and then back to here.
    public function logout()
    {
    }
    
    public function adminmenu()
    {
        return array(
            'main' => 'Portal Login Wiki'
        );
    }
}
